==> kylin-ondevice-nlp-engine_1.0.0.0.orig.tar.gz: kylin-ondevice-nlp-engine-1.0.0.0/test/testnlpservercurl.cpp <==
/*
 * Copyright 2024 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */

#include <iostream>
#include "httplib.h"
#include "json.hpp"
#include "nlpserver.h"
#include "ondevicenlpengine.h"
#include "utils.hpp"

#include <condition_variable>
#include <mutex>
#include <thread>
#include <vector>

struct Answer {
    uint id;
    std::string text;
};
static std::vector<Answer> answers;
std::mutex mutex_answers;
std::condition_variable condition_answers;

int main(int argc, char **argv) {
    std::unique_ptr<httplib::Server> svr;
    svr.reset(new httplib::Server());
    svr->set_default_headers({{"NLP", "ai-subsystem"}});

    // set timeouts and change hostname and port
    svr->set_read_timeout(600);
    svr->set_write_timeout(600);

    if (!svr->bind_to_port("127.0.0.1", 8080)) {
        fprintf(stderr,
                "\ncouldn't bind to server socket: hostname=%s port=%d\n\n",
                "127.0.0.1", 8080);
        return 1;
    }

    const auto handle_completions = [](const httplib::Request &req,
                                       httplib::Response &res) {
        res.set_header("Access-Control-Allow-Origin",
                       req.get_header_value("Origin"));

        bool stream = true;
        json body = json::parse(req.body);
        stream = json_value(body, "stream", true);

        const auto pushText = [&res, stream](const std::string &text) {
            if (!stream) {
                res.set_content(text, "application/json; charset=utf-8");
                return;
            }
            std::unique_lock<std::mutex> lock(mutex_answers);
            json json_text = json::parse(text);
            Answer answer = {json_text["task_id"], text};
            answers.push_back(std::move(answer));
            condition_answers.notify_all();
        };

        ai_engine::lm::nlp::OndeviceNlpEngine *nlpEngine =
            new ai_engine::lm::nlp::OndeviceNlpEngine();
        nlpEngine->initChatModule();
        nlpEngine->setChatResultCallback(pushText);

        const auto chunked_content_provider =
            [nlpEngine](size_t, httplib::DataSink &sink) {
                while (true) {
                    std::unique_lock<std::mutex> lock(mutex_answers);
                    condition_answers.wait(lock,
                                           [&] { return (!answers.empty()); });
                    Answer answer;
                    for (int i = 0; i < (int)answers.size(); i++) {
                        if (answers[i].id == nlpEngine->taskId()) {
                            answer = answers[i];
                            answers.erase(answers.begin() + i);
                        }
                    }
                    // 有可能生成较慢，上面 for 循环未找到对应 id 的回复
                    if (answer.text == "") {
                        continue;
                    }

                    json json_answer = json::parse(answer.text);

                    answer.text = "data: " + answer.text + "\n\n";
                    if (!sink.write(answer.text.c_str(), answer.text.size())) {
                        std::cout << "sink wrinting error" << std::endl;
                        return false;
                    }
                    if (json_answer.contains("stop") &&
                        json_answer["stop"] == true) {
                        break;
                    }
                }
                sink.done();
                return true;
            };

        auto on_complete = [nlpEngine](bool) {
            nlpEngine->stopChat();
            nlpEngine->destroyChatModule();
            delete nlpEngine;
        };

        res.set_chunked_content_provider("text/event-stream",
                                         chunked_content_provider, on_complete);

        std::thread t([nlpEngine, &req]() { nlpEngine->chat(req.body); });

        t.detach();
    };

    svr->Post("/v1/chat/completions", handle_completions);

    svr->new_task_queue = [] { return new httplib::ThreadPool(8); };

    std::thread t([&]() {
        if (!svr->listen_after_bind()) {
            return 1;
        }

        return 0;
    });

    t.join();
    std::cout << "Hello world ! ! !" << std::endl;
    return 0;
}

==> kylin-ondevice-nlp-engine_1.1.0.0.orig.tar.gz: kylin-ondevice-nlp-engine-1.1.0.0/test/testnlpservercurl.cpp <==
/*
 * Copyright 2024 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */

#include <iostream>
#include "httplib.h"
#include "json.hpp"
#include "nlpserver.h"
#include "ondevicenlpengine.h"
#include "utils.hpp"

#include <condition_variable>
#include <mutex>
#include <thread>
#include <vector>

struct Answer {
    uint id;
    std::string text;
};
static std::vector<Answer> answers;
std::mutex mutex_answers;
std::condition_variable condition_answers;
static ai_engine::lm::EngineError error;

int main(int argc, char **argv) {
    std::unique_ptr<httplib::Server> svr;
    svr.reset(new httplib::Server());
    svr->set_default_headers({{"NLP", "ai-subsystem"}});

    // set timeouts and change hostname and port
    svr->set_read_timeout(600);
    svr->set_write_timeout(600);

    if (!svr->bind_to_port("127.0.0.1", 8080)) {
        fprintf(stderr,
                "\ncouldn't bind to server socket: hostname=%s port=%d\n\n",
                "127.0.0.1", 8080);
        return 1;
    }

    const auto handle_completions = [](const httplib::Request &req,
                                       httplib::Response &res) {
        res.set_header("Access-Control-Allow-Origin",
                       req.get_header_value("Origin"));

        bool stream = true;
        json body = json::parse(req.body);
        stream = json_value(body, "stream", true);

        const auto pushText = [&res, stream](ai_engine::lm::nlp::ChatResult result) {
            const std::string &text = result.text;
            if (!stream) {
                res.set_content(text, "application/json; charset=utf-8");
                return;
            }
            std::unique_lock<std::mutex> lock(mutex_answers);
            json json_text = json::parse(text);
            Answer answer = {json_text["task_id"], text};
            answers.push_back(std::move(answer));
            condition_answers.notify_all();
        };

        ai_engine::lm::nlp::OndeviceNlpEngine *nlpEngine =
            new ai_engine::lm::nlp::OndeviceNlpEngine();
        nlpEngine->initChatModule(error);
        nlpEngine->setChatResultCallback(pushText);

        const auto chunked_content_provider =
            [nlpEngine](size_t, httplib::DataSink &sink) {
                while (true) {
                    std::unique_lock<std::mutex> lock(mutex_answers);
                    condition_answers.wait(lock,
                                           [&] { return (!answers.empty()); });
                    Answer answer;
                    for (int i = 0; i < (int)answers.size(); i++) {
                        if (answers[i].id == nlpEngine->taskId()) {
                            answer = answers[i];
                            answers.erase(answers.begin() + i);
                        }
                    }
                    // 有可能生成较慢，上面 for 循环未找到对应 id 的回复
                    if (answer.text == "") {
                        continue;
                    }

                    json json_answer = json::parse(answer.text);

                    answer.text = "data: " + answer.text + "\n\n";
                    if (!sink.write(answer.text.c_str(), answer.text.size())) {
                        std::cout << "sink wrinting error" << std::endl;
                        return false;
                    }
                    if (json_answer.contains("stop") &&
                        json_answer["stop"] == true) {
                        break;
                    }
                }
                sink.done();
                return true;
            };

        auto on_complete = [nlpEngine](bool) {
            nlpEngine->stopChat();
            nlpEngine->destroyChatModule(error);
            delete nlpEngine;
        };

        res.set_chunked_content_provider("text/event-stream",
                                         chunked_content_provider, on_complete);

        std::thread t(
            [nlpEngine, &req]() { nlpEngine->chat(req.body, error); });

        t.detach();
    };

    svr->Post("/v1/chat/completions", handle_completions);

    svr->new_task_queue = [] { return new httplib::ThreadPool(8); };

    std::thread t([&]() {
        if (!svr->listen_after_bind()) {
            return 1;
        }

        return 0;
    });

    t.join();
    std::cout << "Hello world ! ! !" << std::endl;
    return 0;
}
